using IRaCIS.Core.Domain.BaseModel;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Domain;

/// <summary>
/// CRC 提交了 通知QC进行质控
/// </summary>
public class CRCSubmitedAndQCToAuditEvent : DomainEvent
{
    public Guid SubjectVisitId { get; set; }


}

/// <summary>
/// CRC 回复质控质疑 通知QC
/// </summary>
public class CRCRepliedQCChallengeEvent : DomainEvent
{
    public Guid QCChallengeDialogId { get; set; }

    public Guid TrialId { get; set; }

    public Guid SubjectVisitId { get; set; }

}

/// <summary>
/// QC回复 质控质疑，通知CRC
/// </summary>
public class QCRepliedQCChallengeEvent : DomainEvent
{
    public Guid QCChallengeDialogId { get; set; }

    public Guid TrialId { get; set; }

    public Guid SubjectVisitId { get; set; }

}


/// <summary>
/// CRC 回复一致性核查质疑 通知PM
/// </summary>
public class CRCRepliedCheckChallengeEvent : DomainEvent
{
    public Guid CheckChallengeDialogId { get; set; }

    public Guid TrialId { get; set; }

    public Guid SubjectVisitId { get; set; }
}

/// <summary>
/// CRC 回复一致性核查质疑 通知PM
/// </summary>
public class PMRepliedCheckChallengeEvent : DomainEvent
{
    public Guid CheckChallengeDialogId { get; set; }

    public Guid TrialId { get; set; }

    public Guid SubjectVisitId { get; set; }
}

/// <summary>
/// 通知PM 进行一致性核查
/// </summary>
public class CheckStateChangedToAuditEvent : DomainEvent
{
    public Guid SubjectVisitId { get; set; }

}